@@ -25,10 +25,14 @@
 #include "gr-recipe-printer.h"
 #include "gr-recipe-formatter.h"
 #include "gr-ingredients-list.h"
-#include "gr-images.h"
+#include "gr-image.h"
 #include "gr-chef.h"
 #include "gr-recipe-store.h"
+#include "gr-cuisine.h"
+#include "gr-meal.h"
+#include "gr-season.h"
 #include "gr-utils.h"
+#include "gr-convert-units.h"
 
 
 struct _GrRecipePrinter
@@ -108,8 +112,7 @@ begin_print (GtkPrintOperation *operation,
         double page_height;
         GList *page_breaks;
         g_autoptr(GString) s = NULL;
-        g_autoptr(GArray) images = NULL;
-        GrImage *ri;
+        GPtrArray *images;
         PangoRectangle title_rect;
         PangoRectangle left_rect;
         int num_lines;
@@ -121,6 +124,10 @@ begin_print (GtkPrintOperation *operation,
         g_autofree char *instructions = NULL;
         g_autoptr(GrChef) chef = NULL;
         GrRecipeStore *store;
+        PangoLayout *layout;
+        int amount_width;
+        const char *value;
+        g_autofree char *amount = NULL;
 
         store = gr_recipe_store_get ();
         chef = gr_recipe_store_get_chef (store, gr_recipe_get_author (printer->recipe));
@@ -128,11 +135,11 @@ begin_print (GtkPrintOperation *operation,
         width = gtk_print_context_get_width (context);
         height = gtk_print_context_get_height (context);
 
-        g_object_get (printer->recipe, "images", &images, NULL);
+        images = gr_recipe_get_images (printer->recipe);
         if (images && images->len > 0) {
-                int def_index = gr_recipe_get_default_image(printer->recipe);
-                ri = &g_array_index (images, GrImage, def_index);
-                printer->image = load_pixbuf_fit_size (ri->path, width / 2, height / 4, FALSE);
+                int def_index = gr_recipe_get_default_image (printer->recipe);
+                GrImage *ri = g_ptr_array_index (images, def_index);
+                printer->image = gr_image_load_sync (ri, width / 2, height / 4, TRUE);
         }
 
         title_font = pango_font_description_from_string ("Cantarell Bold 18");
@@ -157,36 +164,91 @@ begin_print (GtkPrintOperation *operation,
         g_string_append_printf (s, "%s %s\n", _("Author:"), gr_chef_get_fullname (chef));
         g_string_append_printf (s, "%s %s\n", _("Preparation:"), gr_recipe_get_prep_time (printer->recipe));
         g_string_append_printf (s, "%s %s\n", _("Cooking:"), gr_recipe_get_cook_time (printer->recipe));
-        g_string_append_printf (s, "%s %d\n", _("Serves:"), gr_recipe_get_serves (printer->recipe));
+        amount = gr_number_format (gr_recipe_get_yield (printer->recipe));
+        g_string_append_printf (s, "%s %s %s\n", _("Yield:"), amount, gr_recipe_get_yield_unit (printer->recipe));
+        value = gr_recipe_get_cuisine (printer->recipe);
+        if (value && *value) {
+                const char *title;
+                gr_cuisine_get_data (value, &title, NULL, NULL);
+                g_string_append_printf (s, "%s %s\n", _("Cuisine:"), title);
+        }
+        value = gr_recipe_get_category (printer->recipe);
+        if (value && *value) {
+                g_string_append_printf (s, "%s %s\n", _("Meal:"), gr_meal_get_title (value));
+        }
+        value = gr_recipe_get_season (printer->recipe);
+        if (value && *value) {
+                g_string_append_printf (s, "%s %s\n", _("Season:"), gr_season_get_title (value));
+        }
+
         g_string_append (s, "\n");
 
         pango_layout_set_text (printer->left_layout, s->str, s->len);
 
         g_string_truncate (s, 0);
 
+        ingredients = gr_ingredients_list_new (gr_recipe_get_ingredients (printer->recipe));
+        segs = gr_ingredients_list_get_segments (ingredients);
+
+        layout = gtk_print_context_create_pango_layout (context);
+        pango_layout_set_width (layout, width * PANGO_SCALE);
+        pango_layout_set_font_description (layout, body_font);
+
+        for (j = 0; segs[j]; j++) {
+                ings = gr_ingredients_list_get_ingredients (ingredients, segs[j]);
+                for (i = 0; ings[i]; i++) {
+                        double amount;
+                        GrUnit unit;
+                        double scale = 1.0;
+
+                        unit = gr_ingredients_list_get_unit (ingredients, segs[j], ings[i]);
+                        amount = gr_ingredients_list_get_amount (ingredients, segs[j], ings[i]) * scale;
+                        gr_convert_format (s, amount, unit);
+                }
+        }
+
+        pango_layout_set_text (layout, s->str, s->len);
+        pango_layout_get_size (layout, &amount_width, NULL);
+        g_clear_object (&layout);
+
+        g_string_truncate (s, 0);
+
         printer->bottom_layout = gtk_print_context_create_pango_layout (context);
         pango_layout_set_width (printer->bottom_layout, width * PANGO_SCALE);
         pango_layout_set_font_description (printer->bottom_layout, body_font);
 
-        tabs = pango_tab_array_new (2, FALSE);
-        pango_tab_array_set_tab (tabs, 0, PANGO_TAB_LEFT, 0);
-        pango_tab_array_set_tab (tabs, 1, PANGO_TAB_LEFT, (width / 2) * PANGO_SCALE);
-        pango_layout_set_tabs (printer->bottom_layout, tabs);
-        pango_tab_array_free (tabs);
-
         g_string_append (s, gr_recipe_get_translated_description (printer->recipe));
         g_string_append (s, "\n\n");
 
         attrs = pango_attr_list_new ();
 
-        ingredients = gr_ingredients_list_new (gr_recipe_get_ingredients (printer->recipe));
-        segs = gr_ingredients_list_get_segments (ingredients);
+        value = gr_recipe_get_notes (printer->recipe);
+        if (value && *value) {
+                attr = pango_attr_font_desc_new (title_font);
+                attr->start_index = s->len;
+                g_string_append (s, _("Notes"));
+                attr->end_index = s->len + 1;
+                pango_attr_list_insert (attrs, attr);
+
+                g_string_append (s, "\n");
+                g_string_append (s, value);
+                g_string_append (s, "\n\n");
+        }
+
+        tabs = pango_tab_array_new (2, FALSE);
+        pango_tab_array_set_tab (tabs, 0, PANGO_TAB_LEFT, 0);
+        pango_tab_array_set_tab (tabs, 1, PANGO_TAB_LEFT, amount_width);
+        pango_tab_array_set_tab (tabs, 2, PANGO_TAB_LEFT, (width / 2) * PANGO_SCALE);
+        pango_tab_array_set_tab (tabs, 3, PANGO_TAB_LEFT, (width / 2) * PANGO_SCALE + amount_width);
+        pango_layout_set_tabs (printer->bottom_layout, tabs);
+        pango_tab_array_free (tabs);
+
         for (j = 0; segs[j]; j++) {
                 attr = pango_attr_font_desc_new (title_font);
                 attr->start_index = s->len;
 
                 if (segs[j][0] != 0)
-                        g_string_append (s, segs[j]);
+                        g_string_append (s, g_dgettext (GETTEXT_PACKAGE "-data", segs[j]));
                 else
                         g_string_append (s, _("Ingredients"));
 
@@ -201,34 +263,32 @@ begin_print (GtkPrintOperation *operation,
                         int mid;
                         mid = length / 2 + length % 2;
                         for (i = 0; i < mid; i++) {
-                                char *unit;
+                                g_autofree char *unit = NULL;
 
                                 g_string_append (s, "\n");
 
-                                unit = gr_ingredients_list_scale_unit (ingredients, segs[j], ings[i], 1, 1);
+                                unit = gr_ingredients_list_scale_unit (ingredients, segs[j], ings[i], 1.0);
                                 g_string_append (s, unit);
-                                g_free (unit);
-                                g_string_append (s, " ");
+                                g_clear_pointer (&unit, g_free);
+                                g_string_append (s, "\t");
                                 g_string_append (s, ings[i]);
                                 g_string_append (s, "\t");
                                 if (mid + i < length) {
-                                        unit = gr_ingredients_list_scale_unit (ingredients, segs[j], ings[mid + i], 1, 1);
+                                        unit = gr_ingredients_list_scale_unit (ingredients, segs[j], ings[mid + i], 1.0);
                                         g_string_append (s, unit);
-                                        g_free (unit);
-                                        g_string_append (s, " ");
+                                        g_string_append (s, "\t");
                                         g_string_append (s, ings[mid + i]);
                                 }
                         }
                 }
                 else {
                         for (i = 0; i < length; i++) {
-                                char *unit;
+                                g_autofree char *unit = NULL;
 
                                 g_string_append (s, "\n");
-                                unit = gr_ingredients_list_scale_unit (ingredients, segs[j], ings[i], 1, 1);
+                                unit = gr_ingredients_list_scale_unit (ingredients, segs[j], ings[i], 1.0);
                                 g_string_append (s, unit);
-                                g_free (unit);
-                                g_string_append (s, " ");
+                                g_string_append (s, "\t");
                                 g_string_append (s, ings[i]);
                         }
                 }
@@ -425,6 +485,32 @@ print_done (GtkPrintOperation       *operation,
         g_object_unref (operation);
 }
 
+GFile *
+gr_recipe_printer_get_pdf (GrRecipePrinter *printer,
+                            GrRecipe        *recipe)
+{
+        GtkPrintOperation *operation = NULL;
+        g_autofree char *path = NULL;
+        g_autofree char *name = NULL;
+
+        name = g_strdup (gr_recipe_get_name (GR_RECIPE (recipe)));
+        g_strdelimit (name, "./", ' ');
+        path = g_strdup_printf ("%s/%s.pdf", get_user_data_dir (), name);
+
+        g_set_object (&printer->recipe, recipe);
+        operation = gtk_print_operation_new ();
+
+        g_signal_connect (operation, "begin-print", G_CALLBACK (begin_print), printer);
+        g_signal_connect (operation, "end-print", G_CALLBACK (end_print), printer);
+        g_signal_connect (operation, "draw-page", G_CALLBACK (draw_page), printer);
+        g_signal_connect (operation, "done", G_CALLBACK (print_done), printer);
+        gtk_print_operation_set_allow_async (operation, TRUE);
+        gtk_print_operation_set_export_filename (operation, path);
+
+        gtk_print_operation_run (operation, GTK_PRINT_OPERATION_ACTION_EXPORT, NULL, NULL);
+        return g_file_new_for_path (path);
+}
+
 void
 gr_recipe_printer_print (GrRecipePrinter *printer,
                          GrRecipe        *recipe)
